@@ -706,6 +706,7 @@ func (e *executor) createContainer(containerType string, imageDefinition common.
 			MemorySwap:        e.Config.Docker.GetMemorySwap(),
 			MemoryReservation: e.Config.Docker.GetMemoryReservation(),
 			CpusetCpus:        e.Config.Docker.CPUSetCPUs,
+			CPUShares:         e.Config.Docker.CPUShares,
 			NanoCPUs:          nanoCPUs,
 			Devices:           e.devices,
 			OomKillDisable:    e.Config.Docker.GetOomKillDisable(),
@@ -723,6 +724,7 @@ func (e *executor) createContainer(containerType string, imageDefinition common.
 		NetworkMode:   container.NetworkMode(e.Config.Docker.NetworkMode),
 		Links:         append(e.Config.Docker.Links, e.links...),
 		Binds:         e.volumesManager.Binds(),
+		OomScoreAdj:   e.Config.Docker.OomScoreAdjust,
 		ShmSize:       e.Config.Docker.ShmSize,
 		VolumeDriver:  e.Config.Docker.VolumeDriver,
 		VolumesFrom:   append(e.Config.Docker.VolumesFrom, volumesFrom...),
