@@ -10,7 +10,7 @@ import (
 	"github.com/sirupsen/logrus"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
-	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
+	docker_helpers "gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
 )
 
 type machineProvider struct {
@@ -209,7 +209,7 @@ func (m *machineProvider) remove(machineName string, reason ...interface{}) erro
 
 	details, _ := m.details[machineName]
 	if details == nil {
-		return errors.New("Machine not found")
+		return errors.New("machine not found")
 	}
 
 	details.Reason = fmt.Sprint(reason...)
@@ -234,18 +234,18 @@ func (m *machineProvider) updateMachine(config *common.RunnerConfig, data *machi
 
 	if config.Machine.MaxBuilds > 0 && details.UsedCount >= config.Machine.MaxBuilds {
 		// Limit number of builds
-		return errors.New("Too many builds")
+		return errors.New("too many builds")
 	}
 
 	if data.Total() >= config.Limit && config.Limit > 0 {
 		// Limit maximum number of machines
-		return errors.New("Too many machines")
+		return errors.New("too many machines")
 	}
 
 	if time.Since(details.Used) > time.Second*time.Duration(config.Machine.GetIdleTime()) {
 		if data.Idle >= config.Machine.GetIdleCount() {
 			// Remove machine that are way over the idle time
-			return errors.New("Too many idle machines")
+			return errors.New("too many idle machines")
 		}
 	}
 	return nil
@@ -287,19 +287,55 @@ func (m *machineProvider) createMachines(config *common.RunnerConfig, data *mach
 	}
 }
 
+// intermediateMachineList returns a list of machines that might not yet be
+// persisted on disk, these machines are the ones between being virtually
+// created, and `docker-machine create` getting executed we populate this data
+// set to overcome the race conditions related to not-full set of machines
+// returned by `docker-machine ls -q`
+func (m *machineProvider) intermediateMachineList(excludedMachines []string) []string {
+	var excludedSet map[string]struct{}
+	var intermediateMachines []string
+
+	m.lock.Lock()
+	defer m.lock.Unlock()
+
+	for _, details := range m.details {
+		if details.isPersistedOnDisk() {
+			continue
+		}
+
+		// lazy init set, as most of times we don't create new machines
+		if excludedSet == nil {
+			excludedSet = make(map[string]struct{}, len(excludedMachines))
+			for _, excludedMachine := range excludedMachines {
+				excludedSet[excludedMachine] = struct{}{}
+			}
+		}
+
+		if _, ok := excludedSet[details.Name]; ok {
+			continue
+		}
+
+		intermediateMachines = append(intermediateMachines, details.Name)
+	}
+
+	return intermediateMachines
+}
+
 func (m *machineProvider) loadMachines(config *common.RunnerConfig) (machines []string, err error) {
 	machines, err = m.machine.List()
 	if err != nil {
 		return nil, err
 	}
 
+	machines = append(machines, m.intermediateMachineList(machines)...)
 	machines = filterMachineList(machines, machineFilter(config))
 	return
 }
 
 func (m *machineProvider) Acquire(config *common.RunnerConfig) (data common.ExecutorData, err error) {
 	if config.Machine == nil || config.Machine.MachineName == "" {
-		err = fmt.Errorf("Missing Machine options")
+		err = fmt.Errorf("missing Machine options")
 		return
 	}
 
@@ -335,7 +371,7 @@ func (m *machineProvider) Acquire(config *common.RunnerConfig) (data common.Exec
 
 	// If we have a free machines we can process a build
 	if config.Machine.GetIdleCount() != 0 && machinesData.Idle == 0 {
-		err = errors.New("No free machines that can process builds")
+		err = errors.New("no free machines that can process builds")
 	}
 	return
 }
