@@ -368,7 +368,7 @@ func getRequestJobResponse() (res map[string]interface{}) {
 
 	cache := make([]map[string]interface{}, 1)
 	cache[0] = make(map[string]interface{})
-	cache[0]["key"] = "$CI_COMMIT_REF"
+	cache[0]["key"] = "$CI_COMMIT_SHA"
 	cache[0]["untracked"] = false
 	cache[0]["paths"] = []string{"vendor/*"}
 	cache[0]["policy"] = "push"
